@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22521 $
+ * $Revision: 25159 $
  * $Author: rjongbloed $
- * $Date: 2009-05-06 01:31:23 -0500 (Wed, 06 May 2009) $
+ * $Date: 2011-02-21 23:55:45 -0600 (Mon, 21 Feb 2011) $
  */
 
 #ifndef OPAL_CODEC_OPALPLUGIN_H
@@ -36,6 +36,14 @@
 extern "C" {
 #endif
 
+#ifndef _CRT_NONSTDC_NO_DEPRECATE
+#define _CRT_NONSTDC_NO_DEPRECATE 1
+#endif
+
+#ifndef _CRT_SECURE_NO_WARNINGS
+#define _CRT_SECURE_NO_WARNINGS 1
+#endif
+
 #include <time.h>
 
 #ifdef _MSC_VER
@@ -49,6 +57,10 @@ extern "C" {
 #    define PLUGIN_CODEC_DLL_API __declspec(dllimport)
 #  endif
 
+#if !defined(strcasecmp) && !defined(_WIN32_WCE)
+#define strcasecmp stricmp
+#endif
+
 #else
 
 #define PLUGIN_CODEC_DLL_API
@@ -60,11 +72,14 @@ extern "C" {
 #endif
 #define PWLIB_PLUGIN_API_VERSION 1
 
-#define  PLUGIN_CODEC_VERSION           1    // initial version
+#define  PLUGIN_CODEC_VERSION_FIRST     1    // initial version
 #define  PLUGIN_CODEC_VERSION_WIDEBAND  2    // added wideband
 #define  PLUGIN_CODEC_VERSION_VIDEO     3    // added video
 #define  PLUGIN_CODEC_VERSION_FAX       4    // added fax
 #define  PLUGIN_CODEC_VERSION_OPTIONS   5    // added options handling
+#define  PLUGIN_CODEC_VERSION_INTERSECT 6    // added media option intersection merge functionality
+
+#define  PLUGIN_CODEC_VERSION PLUGIN_CODEC_VERSION_INTERSECT // Always latest version
 
 #define PLUGIN_CODEC_API_VER_FN       PWLibPlugin_GetAPIVersion
 #define PLUGIN_CODEC_API_VER_FN_STR   "PWLibPlugin_GetAPIVersion"
@@ -124,11 +139,11 @@ enum PluginCodec_Flags {
   PluginCodec_MediaTypeFax           = 0x0003,
 
   PluginCodec_InputTypeMask          = 0x0010,
-  PluginCodec_InputTypeRaw           = 0x0000,
+  PluginCodec_InputTypeRaw           = 0x0000, // Note video is always RTP
   PluginCodec_InputTypeRTP           = 0x0010,
 
   PluginCodec_OutputTypeMask         = 0x0020,
-  PluginCodec_OutputTypeRaw          = 0x0000,
+  PluginCodec_OutputTypeRaw          = 0x0000, // Note video is always RTP
   PluginCodec_OutputTypeRTP          = 0x0020,
 
   PluginCodec_RTPTypeMask            = 0x0040,
@@ -161,6 +176,9 @@ enum PluginCodec_Flags {
 
   PluginCodec_BitsPerSamplePos       = 12,
   PluginCodec_BitsPerSampleMask      = 0xf000,
+
+  PluginCodec_ChannelsPos            = 16,
+  PluginCodec_ChannelsMask           = 0x003f0000
 };
 
 enum PluginCodec_CoderFlags {
@@ -169,9 +187,11 @@ enum PluginCodec_CoderFlags {
 };
 
 enum PluginCodec_ReturnCoderFlags {
-  PluginCodec_ReturnCoderLastFrame     = 1,    // indicates when video codec returns last data for frame
-  PluginCodec_ReturnCoderIFrame        = 2,    // indicates when video returns I frame
-  PluginCodec_ReturnCoderRequestIFrame = 4     // indicates when video decoder request I frame for resync
+  PluginCodec_ReturnCoderLastFrame      = 1,    // indicates when video codec returns last data for frame
+  PluginCodec_ReturnCoderIFrame         = 2,    // indicates when video returns I frame
+  PluginCodec_ReturnCoderRequestIFrame  = 4,    // indicates when video decoder request I frame for resync
+  PluginCodec_ReturnCoderBufferTooSmall = 8     // indicates when output buffer is not large enough to receive
+                                                // the data, another call to get_output_data_size is required
 };
 
 struct PluginCodec_Definition;
@@ -187,6 +207,8 @@ struct PluginCodec_Definition;
 #define PLUGINCODEC_CONTROL_TO_CUSTOMISED_OPTIONS "to_customised_options"
 #define PLUGINCODEC_CONTROL_SET_INSTANCE_ID       "set_instance_id"
 #define PLUGINCODEC_CONTROL_SET_LOG_FUNCTION      "set_log_function"
+#define PLUGINCODEC_CONTROL_GET_STATISTICS        "get_statistics"
+#define PLUGINCODEC_CONTROL_TERMINATE_CODEC       "terminate_codec"
 
 
 /* Log function, plug in gets a pointer to this function which allows
@@ -226,7 +248,7 @@ enum PluginCodec_OptionMerge {
   PluginCodec_NotEqualMerge,
   PluginCodec_AlwaysMerge,
   PluginCodec_CustomMerge,
-  PluginCodec_NumOptionMerge,
+  PluginCodec_IntersectionMerge,
 
   PluginCodec_AndMerge = PluginCodec_MinMerge,
   PluginCodec_OrMerge  = PluginCodec_MaxMerge
@@ -264,6 +286,7 @@ struct PluginCodec_Option {
 // Normalised option names
 #define PLUGINCODEC_OPTION_NEEDS_JITTER               "Needs Jitter"
 #define PLUGINCODEC_OPTION_CLOCK_RATE                 "Clock Rate"
+#define PLUGINCODEC_OPTION_CHANNELS                   "Channels"
 #define PLUGINCODEC_OPTION_FRAME_TIME                 "Frame Time"
 #define PLUGINCODEC_OPTION_MAX_FRAME_SIZE             "Max Frame Size"
 #define PLUGINCODEC_OPTION_MAX_BIT_RATE               "Max Bit Rate"
@@ -539,6 +562,7 @@ enum {
 #define PLUGINCODEC_CIF_MPI       "CIF MPI"
 #define PLUGINCODEC_CIF4_MPI     "CIF4 MPI"
 #define PLUGINCODEC_CIF16_MPI   "CIF16 MPI"
+#define PLUGINCODEC_CUSTOM_MPI "Custom MPI"
 
 #define PLUGINCODEC_MPI_DISABLED 33
 
@@ -546,9 +570,13 @@ enum {
 #define PLUGINCODEC_MEDIA_PACKETIZATIONS "Media Packetizations"
 
 #define H261_ANNEX_D "Annex D - Still Image Transmit"
+#define H263_ANNEX_D "Annex D - Unrestricted Motion Vector"
 #define H263_ANNEX_F "Annex F - Advanced Prediction"
 #define H263_ANNEX_I "Annex I - Advanced INTRA Coding"
 #define H263_ANNEX_J "Annex J - Deblocking Filter"
+#define H263_ANNEX_K "Annex K - Slice Structure"
+#define H263_ANNEX_N "Annex N - Reference Picture Selection"
+#define H263_ANNEX_S "Annex S - Alternative INTER VLC"
 #define H263_ANNEX_T "Annex T - Modified Quantization"
 
 #ifndef STRINGIZE
