@@ -48,13 +48,12 @@
 
 #ifndef __H263P_1998_H__
 #define __H263P_1998_H__ 1
+
 #include <codec/opalplugin.h>
 #include "h263pframe.h"
 #include "rfc2190.h"
 #include "critsect.h"
 
-#include <stdio.h>
-
 #if TRACE_FILE
 #include "tracer.h"
 #endif
@@ -130,6 +129,8 @@ class H263_Base_EncoderContext
 
     virtual void SetMaxRTPFrameSize (unsigned size) = 0;
 
+    int m_targetBitRate;
+
   protected:
     virtual bool InitContext() = 0;
 
