@@ -21,18 +21,23 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22253 $
+ * $Revision: 24766 $
  * $Author: rjongbloed $
- * $Date: 2009-03-23 19:37:48 -0500 (Mon, 23 Mar 2009) $
+ * $Date: 2010-09-30 02:18:52 -0500 (Thu, 30 Sep 2010) $
  */
 
 #ifndef _IvrOPAL_MAIN_H
 #define _IvrOPAL_MAIN_H
 
+#if OPAL_IVR
+#else
+#error Cannot compile IVR test program without OPAL_IVR set!
+#endif
 
-class MyManager : public OpalManager
+
+class MyManager : public OpalManagerConsole
 {
-    PCLASSINFO(MyManager, OpalManager)
+    PCLASSINFO(MyManager, OpalManagerConsole)
 
   public:
     virtual void OnClearedCall(OpalCall & call); // Callback override
@@ -41,25 +46,12 @@ class MyManager : public OpalManager
 };
 
 
-class MySIPEndPoint : public SIPEndPoint
-{
-    PCLASSINFO(MySIPEndPoint, SIPEndPoint)
-
-  public:
-    MySIPEndPoint(OpalManager & manager) : SIPEndPoint(manager) { }
-    void OnRegistrationStatus(const RegistrationStatus & status);
-
-    PSyncPoint m_completed;
-};
-
-
 class IvrOPAL : public PProcess
 {
     PCLASSINFO(IvrOPAL, PProcess)
 
   public:
     IvrOPAL();
-    ~IvrOPAL();
 
     virtual void Main();
     virtual bool OnInterrupt(bool);
