@@ -24,24 +24,26 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22664 $
+ * $Revision: 26023 $
  * $Author: rjongbloed $
- * $Date: 2009-05-19 05:23:00 -0500 (Tue, 19 May 2009) $
+ * $Date: 2011-06-14 03:37:04 -0500 (Tue, 14 Jun 2011) $
  */
 
 #ifndef _OpenPhone_VERSION_H
 #define _OpenPhone_VERSION_H
 
+#include <opal/buildopts.h>
+
 #define	PRODUCT_NAME_TEXT	"Open Phone"
 #define	EXE_NAME_TEXT	        "openphone"
 #define	MANUFACTURER_TEXT	"Vox Lucida"
 #define	COPYRIGHT_HOLDER	"Vox Lucida Pty. Ltd. (Australia)"
 
 
-#define MAJOR_VERSION 3
-#define MINOR_VERSION 6
-#define BUILD_TYPE    BetaCode
-#define BUILD_NUMBER 2
+#define MAJOR_VERSION OPAL_MAJOR
+#define MINOR_VERSION OPAL_MINOR
+#define BUILD_TYPE    ReleaseCode
+#define BUILD_NUMBER  OPAL_BUILD
 
 
 #endif  // _OpenPhone_VERSION_H
