@@ -22,9 +22,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23863 $
+ * $Revision: 25300 $
  * $Author: rjongbloed $
- * $Date: 2009-12-09 20:42:20 -0600 (Wed, 09 Dec 2009) $
+ * $Date: 2011-03-09 13:22:53 -0600 (Wed, 09 Mar 2011) $
  */
 
 #include <ptlib.h>
@@ -123,7 +123,7 @@ OpalRFC2833Proto::OpalRFC2833Proto(OpalRTPConnection & conn, const PNotifier & r
 #ifdef _MSC_VER
 #pragma warning(disable:4355)
 #endif
-  , m_receiveHandler(PCREATE_NOTIFIER(ReceivedPacket))
+  , m_receiveHandler(PCREATE_RTPFilterNotifier(ReceivedPacket))
 #ifdef _MSC_VER
 #pragma warning(default:4355)
 #endif
@@ -150,8 +150,6 @@ OpalRFC2833Proto::OpalRFC2833Proto(OpalRTPConnection & conn, const PNotifier & r
 
 OpalRFC2833Proto::~OpalRFC2833Proto()
 {
-  if (m_rtpSession != NULL)
-    m_connection.ReleaseSession(m_rtpSession->GetSessionID());
 }
 
 PBoolean OpalRFC2833Proto::SendToneAsync(char tone, unsigned duration)
@@ -167,22 +165,19 @@ PBoolean OpalRFC2833Proto::SendToneAsync(char tone, unsigned duration)
     }
   }
 
+  // convert tone to correct code
+  PINDEX code = ASCIIToRFC2833(tone, m_txCapabilitySet[NSECodeBase]);
+  if (code == P_MAX_INDEX || !m_txCapabilitySet[code])
+    return false; // Silently ignore illegal tone values for this instance
+
   // if transmittter is ever in this state, then stop the duration timer
   if (m_payloadType == RTP_DataFrame::IllegalPayloadType) {
     PTRACE(2, "RFC2833\tNo payload type, cannot send packet.");
     return false;
   }
 
-  // convert tone to correct code
-  PINDEX code = ASCIIToRFC2833(tone, m_txCapabilitySet[NSECodeBase]);
-
   // if same tone as last time and still transmitting, just extend the time
-  if (m_transmitState == TransmitIdle || (code != ' ' && code != m_transmitCode)) {
-    if (code == P_MAX_INDEX || !m_txCapabilitySet[code]) {
-      m_transmitState = TransmitIdle;
-      return false;
-    }
-
+  if (m_transmitState == TransmitIdle || (tone != ' ' && code != m_transmitCode)) {
     // kick off the transmitter
     m_transmitCode             = (BYTE)code;
     m_transmitState            = TransmitActive;
@@ -299,8 +294,12 @@ void OpalRFC2833Proto::SendAsyncFrame()
   } 
 
   PTRACE(frame.GetMarker() ? 3 : 4,
-         "RFC2833\tSent " << ((payload[1] & 0x80) ? "end" : "tone") << ": code=" << (unsigned)m_transmitCode <<
-         ", dur=" << m_transmitDuration << ", ts=" << frame.GetTimestamp() << ", mkr=" << frame.GetMarker());
+         "RFC2833\tSent " << ((payload[1] & 0x80) ? "end" : "tone") << ": "
+         "code=" << (unsigned)m_transmitCode << ", "
+         "dur=" << m_transmitDuration << ", "
+         "ts=" << frame.GetTimestamp() << ", "
+         "mkr=" << frame.GetMarker() << ", "
+         "pt=" << m_payloadType);
 }
 
 
@@ -390,11 +389,13 @@ void OpalRFC2833Proto::OnEndReceive(char tone, unsigned duration, unsigned times
 }
 
 
-void OpalRFC2833Proto::ReceivedPacket(RTP_DataFrame & frame, INT)
+void OpalRFC2833Proto::ReceivedPacket(RTP_DataFrame & frame, RTP_Session::SendReceiveStatus & status)
 {
   if (frame.GetPayloadType() != m_payloadType || frame.GetPayloadSize() == 0)
     return;
 
+  status = RTP_Session::e_IgnorePacket;
+
   PWaitAndSignal mutex(m_mutex);
 
   if (frame.GetPayloadSize() < 4) {
