@@ -26,9 +26,9 @@
  *
  * Contributor(s): Many thanks to Simon Horne.
  *
- * $Revision: 23224 $
+ * $Revision: 25368 $
  * $Author: rjongbloed $
- * $Date: 2009-08-19 23:26:44 -0500 (Wed, 19 Aug 2009) $
+ * $Date: 2011-03-21 01:34:07 -0500 (Mon, 21 Mar 2011) $
  */
 
 #include <ptlib.h>
@@ -57,7 +57,7 @@
 /////////////////////////////////////////////////////////////////////////////
 
 H323EndPoint::H323EndPoint(OpalManager & manager)
-  : OpalRTPEndPoint(manager, "h323", CanTerminateCall)
+  : OpalRTPEndPoint(manager, "h323", CanTerminateCall|SupportsE164)
   , autoCallForward(true)
   , disableFastStart(false)
   , disableH245Tunneling(false)
@@ -774,7 +774,7 @@ PBoolean H323EndPoint::ParsePartyName(const PString & remoteParty,
     if (hostname.IsEmpty()) {
       PString username = url.GetUserName();
       // make sure the number has only digits and +
-      if (username.FindSpan("+0123456789") == P_MAX_INDEX) {
+      if (OpalIsE164(username)) {
         PString newName;
         if (PDNS::ENUMLookup(username, "E2U+h323", newName)) {
           PTRACE(4, "H323\tENUM converted remote party " << username << " to " << newName);
@@ -1087,12 +1087,6 @@ PBoolean H323EndPoint::OnOutgoingCall(H323Connection & /*connection*/,
 }
 
 
-void H323EndPoint::OnConnectionCleared(H323Connection & /*connection*/,
-                                       const PString & /*token*/)
-{
-}
-
-
 #if PTRACING
 static void OnStartStopChannel(const char * startstop, const H323Channel & channel)
 {
