@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24130 $
+ * $Revision: 25924 $
  * $Author: rjongbloed $
- * $Date: 2010-03-17 02:06:06 -0500 (Wed, 17 Mar 2010) $
+ * $Date: 2011-05-30 18:15:54 -0500 (Mon, 30 May 2011) $
  */
 
 #include <ptlib.h>
@@ -56,6 +56,44 @@
 class OpalManager_C;
 
 
+ostream & operator<<(ostream & strm, OpalMessageType type)
+{
+  static const char * Types[] = {
+    "IndCommandError",
+    "CmdSetGeneralParameters",
+    "CmdSetProtocolParameters",
+    "CmdRegistration",
+    "IndRegistration",
+    "CmdSetUpCall",
+    "IndIncomingCall",
+    "CmdAnswerCall",
+    "CmdClearCall",
+    "IndAlerting",
+    "IndEstablished",
+    "IndUserInput",
+    "IndCallCleared",
+    "CmdHoldCall",
+    "CmdRetrieveCall",
+    "CmdTransferCall",
+    "CmdUserInput",
+    "IndMessageWaiting",
+    "IndMediaStream",
+    "CmdMediaStream",
+    "CmdSetUserData",
+    "IndLineAppearance",
+    "CmdStartRecording",
+    "CmdStopRecording",
+    "IndProceeding",
+    "CmdAlerting",
+    "IndOnHold",
+    "IndOffHold",
+    "IndTransferCall",
+    "IndCompletedIVR"
+  };
+  return strm << Types[type];
+}
+
+
 ostream & operator<<(ostream & strm, OpalRegistrationStates state)
 {
   static const char * States[] = { "Successful", "Removed", "Failed", "Retrying", "Restored" };
@@ -93,7 +131,6 @@ class OpalMessageBuffer
 #define SET_MESSAGE_STRING(msg, member, str) (msg).SetString(&(msg)->member, str)
 
 
-#if OPAL_PTLIB_AUDIO
 #if OPAL_HAS_PCSS
 
 class OpalPCSSEndPoint_C : public OpalPCSSEndPoint
@@ -108,8 +145,7 @@ class OpalPCSSEndPoint_C : public OpalPCSSEndPoint
     OpalManager_C & m_manager;
 };
 
-#endif
-#endif // OPAL_PTLIB_AUDIO
+#endif // OPAL_HAS_PCSS
 
 
 class OpalLocalEndPoint_C : public OpalLocalEndPoint
@@ -135,6 +171,23 @@ class OpalLocalEndPoint_C : public OpalLocalEndPoint
 };
 
 
+#if OPAL_IVR
+
+class OpalIVREndPoint_C : public OpalIVREndPoint
+{
+  public:
+    OpalIVREndPoint_C(OpalManager_C & manager);
+
+    virtual bool OnIncomingCall(OpalLocalConnection & connection);
+    virtual void OnEndDialog(OpalIVRConnection & connection);
+
+  private:
+    OpalManager_C & m_manager;
+};
+
+#endif // OPAL_IVR
+
+
 #if OPAL_SIP
 class SIPEndPoint_C : public SIPEndPoint
 {
@@ -165,11 +218,12 @@ class OpalManager_C : public OpalManager
 {
   public:
     OpalManager_C(unsigned version)
-      : localEP(NULL)
-#if OPAL_PTLIB_AUDIO
+      : m_localEP(NULL)
 #if OPAL_HAS_PCSS
-      , pcssEP(NULL)
+      , m_pcssEP(NULL)
 #endif
+#if OPAL_IVR
+      , m_ivrEP(NULL)
 #endif
       , m_apiVersion(version)
       , m_manualAlerting(false)
@@ -189,6 +243,8 @@ class OpalManager_C : public OpalManager
     OpalMessage * SendMessage(const OpalMessage * message);
 
     virtual void OnEstablishedCall(OpalCall & call);
+    virtual void OnHold(OpalConnection & connection, bool fromRemote, bool onHold);
+    virtual bool OnTransferNotify(OpalConnection &, const PStringToString &);
     virtual PBoolean OnOpenMediaStream(OpalConnection & connection, OpalMediaStream & stream);
     virtual void OnClosedMediaStream(const OpalMediaStream & stream);
     virtual void OnUserInputString(OpalConnection & connection, const PString & value);
@@ -197,7 +253,7 @@ class OpalManager_C : public OpalManager
     virtual void OnProceeding(OpalConnection & conenction);
     virtual void OnClearedCall(OpalCall & call);
 
-    bool IsManualAlerting() const { return m_manualAlerting; }
+    void SendIncomingCallInfo(const OpalConnection & connection);
 
   private:
     void HandleSetGeneral    (const OpalMessage & message, OpalMessageBuffer & response);
@@ -220,11 +276,12 @@ class OpalManager_C : public OpalManager
 
     bool FindCall(const char * token, OpalMessageBuffer & response, PSafePtr<OpalCall> & call);
 
-    OpalLocalEndPoint_C * localEP;
-#if OPAL_PTLIB_AUDIO
+    OpalLocalEndPoint_C * m_localEP;
 #if OPAL_HAS_PCSS
-    OpalPCSSEndPoint_C  * pcssEP;
+    OpalPCSSEndPoint_C  * m_pcssEP;
 #endif
+#if OPAL_IVR
+    OpalIVREndPoint_C   * m_ivrEP;
 #endif
 
     unsigned                  m_apiVersion;
@@ -270,16 +327,13 @@ public:
       traceOpts |= PTrace::AppendToFile;
 
     PTrace::Initialise(level, filename, traceOpts);
-    PTRACE(3, "OpalC\tStart Up.");
+    PTRACE(1, "OpalC\tStart Up, OPAL version " << OpalGetVersion());
 #endif
   }
 
   ~PProcess_C()
   {
-#if PTRACING
-    PTRACE(3, "OpalC\tShut Down.");
-    PTrace::SetStream(NULL);
-#endif
+    PTRACE(1, "OpalC\tShut Down.");
   }
 
 private:
@@ -388,7 +442,6 @@ OpalLocalEndPoint_C::OpalLocalEndPoint_C(OpalManager_C & mgr)
   , m_mediaTiming(OpalMediaTimingSynchronous)
   , m_manager(mgr)
 {
-  m_deferredAlerting = mgr.IsManualAlerting();
 }
 
 
@@ -414,8 +467,10 @@ bool OpalLocalEndPoint_C::OnOutgoingCall(const OpalLocalConnection & connection)
 }
 
 
-static void SetIncomingCallInfo(OpalMessageBuffer & message, const OpalConnection & connection)
+void OpalManager_C::SendIncomingCallInfo(const OpalConnection & connection)
 {
+  OpalMessageBuffer message(OpalIndIncomingCall);
+
   PSafePtr<OpalConnection> network = connection.GetOtherPartyConnection();
   PAssert(network != NULL, PLogicError); // Should not happen!
 
@@ -427,7 +482,18 @@ static void SetIncomingCallInfo(OpalMessageBuffer & message, const OpalConnectio
   SET_MESSAGE_STRING(message, m_param.m_incomingCall.m_calledAddress, network->GetCalledPartyURL());
   SET_MESSAGE_STRING(message, m_param.m_incomingCall.m_calledPartyNumber, network->GetCalledPartyNumber());
 
-  const OpalProductInfo & info = connection.GetProductInfo();
+  if (m_apiVersion >= 22) {
+    PString redirect = network->GetRedirectingParty();
+    SET_MESSAGE_STRING(message, m_param.m_incomingCall.m_referredByAddress, redirect);
+    if (!OpalIsE164(redirect)) {
+      redirect = PURL(redirect).GetUserName();
+      if (!OpalIsE164(redirect))
+        redirect.MakeEmpty();
+    }
+    SET_MESSAGE_STRING(message, m_param.m_incomingCall.m_redirectingNumber, redirect);
+  }
+
+  const OpalProductInfo & info = network->GetRemoteProductInfo();
   SET_MESSAGE_STRING(message, m_param.m_incomingCall.m_product.m_vendor,  info.vendor);
   SET_MESSAGE_STRING(message, m_param.m_incomingCall.m_product.m_name,    BuildProductName(info));
   SET_MESSAGE_STRING(message, m_param.m_incomingCall.m_product.m_version, info.version);
@@ -448,14 +514,14 @@ static void SetIncomingCallInfo(OpalMessageBuffer & message, const OpalConnectio
                     " E.164=\"" << message->m_param.m_incomingCall.m_calledPartyNumber << "\"\n"
             "  AlertingType=\"" << message->m_param.m_incomingCall.m_alertingType << "\"\n"
             "        CallID=\"" << message->m_param.m_incomingCall.m_protocolCallId << '"');
+
+  PostMessage(message);
 }
 
 
 bool OpalLocalEndPoint_C::OnIncomingCall(OpalLocalConnection & connection)
 {
-  OpalMessageBuffer message(OpalIndIncomingCall);
-  SetIncomingCallInfo(message, connection);
-  m_manager.PostMessage(message);
+  m_manager.SendIncomingCallInfo(connection);
   return true;
 }
 
@@ -564,22 +630,18 @@ bool OpalLocalEndPoint_C::IsSynchronous() const
 
 ///////////////////////////////////////
 
-#if OPAL_PTLIB_AUDIO
 #if OPAL_HAS_PCSS
 
 OpalPCSSEndPoint_C::OpalPCSSEndPoint_C(OpalManager_C & mgr)
   : OpalPCSSEndPoint(mgr)
   , m_manager(mgr)
 {
-  m_deferredAlerting = mgr.IsManualAlerting();
 }
 
 
 PBoolean OpalPCSSEndPoint_C::OnShowIncoming(const OpalPCSSConnection & connection)
 {
-  OpalMessageBuffer message(OpalIndIncomingCall);
-  SetIncomingCallInfo(message, connection);
-  m_manager.PostMessage(message);
+  m_manager.SendIncomingCallInfo(connection);
   return true;
 }
 
@@ -592,10 +654,48 @@ PBoolean OpalPCSSEndPoint_C::OnShowOutgoing(const OpalPCSSConnection & connectio
   return true;
 }
 
+#endif // OPAL_HAS_PCSS
+
+
+///////////////////////////////////////
+
+#if OPAL_IVR
+
+OpalIVREndPoint_C::OpalIVREndPoint_C(OpalManager_C & manager)
+  : OpalIVREndPoint(manager)
+  , m_manager(manager)
+{
+}
+
+
+bool OpalIVREndPoint_C::OnIncomingCall(OpalLocalConnection & connection)
+{
+  m_manager.SendIncomingCallInfo(connection);
+  return true;
+}
+
+
+void OpalIVREndPoint_C::OnEndDialog(OpalIVRConnection & connection)
+{
+  PTRACE(4, "OpalC API\tOnEndDialog for " << connection);
+
+  // Do not call ancestor and start a long pause, as do not want it to hang up
+  connection.TransferConnection("silence=3600000"); 
+
+  // Send message to app, which may (or may not) start a new IVR script
+  OpalMessageBuffer message(OpalIndCompletedIVR);
+  SET_MESSAGE_STRING(message, m_param.m_ivrStatus.m_callToken, connection.GetCall().GetToken());
+
+  PStringStream varStr;
+  varStr << connection.GetVXMLSession().GetVariables();
+  SET_MESSAGE_STRING(message, m_param.m_ivrStatus.m_variables, varStr);
+
+  m_manager.PostMessage(message);
+}
+
 #endif
-#endif // OPAL_PTLIB_AUDIO
-
 
+    
 ///////////////////////////////////////
 
 #if OPAL_SIP
@@ -762,9 +862,9 @@ bool OpalManager_C::Initialise(const PCaselessString & options)
   }
 
   PINDEX t38Pos = options.Find("t38");
-  if (t38Pos < defProtoPos) {
+  if (t38Pos < defUserPos) {
     defUser = "t38:";
-    defProtoPos = t38Pos;
+    defUserPos = t38Pos;
   }
 #endif
 
@@ -782,9 +882,10 @@ bool OpalManager_C::Initialise(const PCaselessString & options)
 
 
 #if OPAL_IVR
-  if (options.Find("ivr") != P_MAX_INDEX) {
-    new OpalIVREndPoint(*this);
-    AddRouteEntry(".*:#=ivr:"); // A hash from anywhere goes to IVR
+  PINDEX ivrPos = options.Find("ivr");
+  if (ivrPos < defUserPos) {
+    defUser = "ivr:";
+    defUserPos = localPos;
   }
 #endif
 
@@ -831,20 +932,25 @@ bool OpalManager_C::Initialise(const PCaselessString & options)
   }
 #endif
 
-#if OPAL_PTLIB_AUDIO
 #if OPAL_HAS_PCSS
   if (pcPos != P_MAX_INDEX) {
-    pcssEP = new OpalPCSSEndPoint_C(*this);
+    m_pcssEP = new OpalPCSSEndPoint_C(*this);
     AddRouteEntry("pc:.*=" + defProto + ":<da>");
   }
-#endif
 #endif
 
   if (localPos != P_MAX_INDEX) {
-    localEP = new OpalLocalEndPoint_C(*this);
+    m_localEP = new OpalLocalEndPoint_C(*this);
     AddRouteEntry("local:.*=" + defProto + ":<da>");
   }
 
+#if OPAL_IVR
+  if (ivrPos != P_MAX_INDEX) {
+    m_ivrEP = new OpalIVREndPoint_C(*this);
+    AddRouteEntry("ivr:.*=" + defProto + ":<da>");
+  }
+#endif
+
   return true;
 }
 
@@ -884,6 +990,8 @@ OpalMessage * OpalManager_C::SendMessage(const OpalMessage * message)
   if (message == NULL)
     return NULL;
 
+  PTRACE(4, "OpalC API\tHandling message " << message->m_type << " from application");
+
   OpalMessageBuffer response(message->m_type);
 
   switch (message->m_type) {
@@ -899,6 +1007,9 @@ OpalMessage * OpalManager_C::SendMessage(const OpalMessage * message)
     case OpalCmdSetUpCall :
       HandleSetUpCall(*message, response);
       break;
+    case OpalCmdAlerting :
+      HandleAlerting(*message, response);
+      break;
     case OpalCmdAnswerCall :
       HandleAnswerCall(*message, response);
       break;
@@ -939,19 +1050,17 @@ OpalMessage * OpalManager_C::SendMessage(const OpalMessage * message)
 
 void OpalManager_C::HandleSetGeneral(const OpalMessage & command, OpalMessageBuffer & response)
 {
-#if OPAL_PTLIB_AUDIO
 #if OPAL_HAS_PCSS
-  if (pcssEP != NULL) {
-    SET_MESSAGE_STRING(response, m_param.m_general.m_audioRecordDevice, pcssEP->GetSoundChannelRecordDevice());
+  if (m_pcssEP != NULL) {
+    SET_MESSAGE_STRING(response, m_param.m_general.m_audioRecordDevice, m_pcssEP->GetSoundChannelRecordDevice());
     if (!IsNullString(command.m_param.m_general.m_audioRecordDevice))
-      pcssEP->SetSoundChannelRecordDevice(command.m_param.m_general.m_audioRecordDevice);
+      m_pcssEP->SetSoundChannelRecordDevice(command.m_param.m_general.m_audioRecordDevice);
 
-    SET_MESSAGE_STRING(response, m_param.m_general.m_audioPlayerDevice, pcssEP->GetSoundChannelPlayDevice());
+    SET_MESSAGE_STRING(response, m_param.m_general.m_audioPlayerDevice, m_pcssEP->GetSoundChannelPlayDevice());
     if (!IsNullString(command.m_param.m_general.m_audioPlayerDevice))
-      pcssEP->SetSoundChannelPlayDevice(command.m_param.m_general.m_audioPlayerDevice);
+      m_pcssEP->SetSoundChannelPlayDevice(command.m_param.m_general.m_audioPlayerDevice);
   }
-#endif
-#endif // OPAL_PTLIB_AUDIO
+#endif // OPAL_HAS_PCSS
 
 #if OPAL_VIDEO
   PVideoDevice::OpenArgs video = GetVideoInputDevice();
@@ -1007,11 +1116,17 @@ void OpalManager_C::HandleSetGeneral(const OpalMessage & command, OpalMessageBuf
     PString autoXxMedia;
     if (autoStart == OpalMediaType::Receive) {
       SET_MESSAGE_STRING(response, m_param.m_general.m_autoRxMedia, strm);
-      autoXxMedia = command.m_param.m_general.m_autoRxMedia;
+      if (command.m_param.m_general.m_autoRxMedia != NULL)
+        autoXxMedia = command.m_param.m_general.m_autoRxMedia;
+      else
+        autoXxMedia = strm;
     }
     else {
       SET_MESSAGE_STRING(response, m_param.m_general.m_autoTxMedia, strm);
-      autoXxMedia = command.m_param.m_general.m_autoTxMedia;
+      if (command.m_param.m_general.m_autoTxMedia != NULL)
+        autoXxMedia = command.m_param.m_general.m_autoTxMedia;
+      else
+        autoXxMedia = strm;
     }
 
     PStringArray enabledMediaTypes = autoXxMedia.Tokenise(" \t\n", false);
@@ -1055,9 +1170,9 @@ void OpalManager_C::HandleSetGeneral(const OpalMessage & command, OpalMessageBuf
   if (command.m_param.m_general.m_rtpPortBase != 0)
     SetRtpIpPorts(command.m_param.m_general.m_rtpPortBase, command.m_param.m_general.m_rtpPortMax);
 
-  response->m_param.m_general.m_rtpTypeOfService = GetRtpIpTypeofService();
+  response->m_param.m_general.m_rtpTypeOfService = GetMediaTypeOfService();
   if (command.m_param.m_general.m_rtpTypeOfService != 0)
-    SetRtpIpTypeofService(command.m_param.m_general.m_rtpTypeOfService);
+    SetMediaTypeOfService(command.m_param.m_general.m_rtpTypeOfService);
 
   response->m_param.m_general.m_rtpMaxPayloadSize = GetMaxRtpPayloadSize();
   if (command.m_param.m_general.m_rtpMaxPayloadSize != 0)
@@ -1100,36 +1215,34 @@ void OpalManager_C::HandleSetGeneral(const OpalMessage & command, OpalMessageBuf
   if (m_apiVersion < 3)
     return;
 
-#if OPAL_PTLIB_AUDIO
 #if OPAL_HAS_PCSS
-  if (pcssEP != NULL) {
-    response->m_param.m_general.m_audioBuffers = pcssEP->GetSoundChannelBufferDepth();
+  if (m_pcssEP != NULL) {
+    response->m_param.m_general.m_audioBuffers = m_pcssEP->GetSoundChannelBufferDepth();
     if (command.m_param.m_general.m_audioBuffers != 0)
-      pcssEP->SetSoundChannelBufferDepth(command.m_param.m_general.m_audioBuffers);
+      m_pcssEP->SetSoundChannelBufferDepth(command.m_param.m_general.m_audioBuffers);
   }
-#endif
 #endif
 
   if (m_apiVersion < 5)
     return;
 
-  if (localEP != NULL) {
-    response->m_param.m_general.m_mediaReadData = localEP->m_mediaReadData;
+  if (m_localEP != NULL) {
+    response->m_param.m_general.m_mediaReadData = m_localEP->m_mediaReadData;
     if (command.m_param.m_general.m_mediaReadData != NULL)
-      localEP->m_mediaReadData = command.m_param.m_general.m_mediaReadData;
+      m_localEP->m_mediaReadData = command.m_param.m_general.m_mediaReadData;
 
-    response->m_param.m_general.m_mediaWriteData = localEP->m_mediaWriteData;
+    response->m_param.m_general.m_mediaWriteData = m_localEP->m_mediaWriteData;
     if (command.m_param.m_general.m_mediaWriteData != NULL)
-      localEP->m_mediaWriteData = command.m_param.m_general.m_mediaWriteData;
+      m_localEP->m_mediaWriteData = command.m_param.m_general.m_mediaWriteData;
 
-    response->m_param.m_general.m_mediaDataHeader = localEP->m_mediaDataHeader;
+    response->m_param.m_general.m_mediaDataHeader = m_localEP->m_mediaDataHeader;
     if (command.m_param.m_general.m_mediaDataHeader != 0)
-      localEP->m_mediaDataHeader = command.m_param.m_general.m_mediaDataHeader;
+      m_localEP->m_mediaDataHeader = command.m_param.m_general.m_mediaDataHeader;
 
     if (m_apiVersion >= 20) {
-      response->m_param.m_general.m_mediaTiming = localEP->m_mediaTiming;
+      response->m_param.m_general.m_mediaTiming = m_localEP->m_mediaTiming;
       if (command.m_param.m_general.m_mediaTiming != 0)
-        localEP->m_mediaTiming = command.m_param.m_general.m_mediaTiming;
+        m_localEP->m_mediaTiming = command.m_param.m_general.m_mediaTiming;
     }
   }
 
@@ -1211,22 +1324,31 @@ void OpalManager_C::HandleSetGeneral(const OpalMessage & command, OpalMessageBuf
   if (m_apiVersion < 17)
     return;
 
-#if OPAL_PTLIB_AUDIO
 #if OPAL_HAS_PCSS
-  if (pcssEP != NULL) {
-    response->m_param.m_general.m_audioBufferTime = pcssEP->GetSoundChannelBufferTime();
+  if (m_pcssEP != NULL) {
+    response->m_param.m_general.m_audioBufferTime = m_pcssEP->GetSoundChannelBufferTime();
     if (command.m_param.m_general.m_audioBufferTime != 0)
-      pcssEP->SetSoundChannelBufferTime(command.m_param.m_general.m_audioBufferTime);
+      m_pcssEP->SetSoundChannelBufferTime(command.m_param.m_general.m_audioBufferTime);
   }
-#endif
 #endif
 
   if (m_apiVersion < 19)
     return;
 
-  response->m_param.m_general.m_audioBufferTime = m_manualAlerting ? 2 : 1;
-  if (command.m_param.m_general.m_manualAlerting != 0)
+  response->m_param.m_general.m_manualAlerting = m_manualAlerting ? 2 : 1;
+  if (command.m_param.m_general.m_manualAlerting != 0) {
     m_manualAlerting = command.m_param.m_general.m_manualAlerting != 1;
+    if (m_localEP)
+      m_localEP->SetDeferredAlerting(m_manualAlerting);
+#if OPAL_HAS_PCSS
+    if (m_pcssEP != NULL)
+      m_pcssEP->SetDeferredAlerting(m_manualAlerting);
+#endif
+#if OPAL_IVR
+    if (m_ivrEP != NULL)
+      m_ivrEP->SetDeferredAlerting(m_manualAlerting);
+#endif
+  }
 }
 
 
@@ -1330,8 +1452,44 @@ void OpalManager_C::HandleSetProtocol(const OpalMessage & command, OpalMessageBu
   FillOpalProductInfo(command, response, product);
   ep->SetProductInfo(product);
 
+#if OPAL_IVR
+  OpalIVREndPoint * ivr = dynamic_cast<OpalIVREndPoint *>(ep);
+  if (ivr != NULL)
+    ivr->SetDefaultVXML(command.m_param.m_protocol.m_interfaceAddresses);
+  else
+#endif
   if (command.m_param.m_protocol.m_interfaceAddresses != NULL)
     StartStopListeners(ep, command.m_param.m_protocol.m_interfaceAddresses, response);
+
+  if (m_apiVersion < 22)
+    return;
+
+  unsigned mode = ep->GetSendUserInputMode();
+  if (mode != OpalConnection::SendUserInputAsProtocolDefault)
+    ++mode;
+  else
+    mode = OpalUserInputDefault;
+  response->m_param.m_protocol.m_userInputMode = (OpalUserInputModes)mode;
+
+  mode = command.m_param.m_protocol.m_userInputMode;
+  if (mode != OpalUserInputDefault && mode <= OpalConnection::NumSendUserInputModes)
+    --mode;
+  else
+    mode = OpalConnection::SendUserInputAsProtocolDefault;
+  ep->SetSendUserInputMode((OpalConnection::SendUserInputModes)mode);
+
+  if (m_apiVersion < 23)
+    return;
+
+  PStringStream strm;
+  strm << ep->GetDefaultStringOptions();
+  SET_MESSAGE_STRING(response, m_param.m_protocol.m_defaultOptions, strm);
+  if (!IsNullString(command.m_param.m_protocol.m_defaultOptions)) {
+    OpalConnection::StringOptions newOptions;
+    strm = command.m_param.m_protocol.m_defaultOptions;
+    strm >> newOptions;
+    ep->SetDefaultStringOptions(newOptions);
+  }
 }
 
 
@@ -1415,7 +1573,7 @@ void OpalManager_C::HandleRegistration(const OpalMessage & command, OpalMessageB
         subParams.m_expire = m_apiVersion = command.m_param.m_registrationInfo.m_timeToLive;
 #else
         subParams.m_expire = m_apiVersion >= 13 ? command.m_param.m_registrationInfo.m_timeToLive
-                                               : (unsigned)command.m_param.m_registrationInfo.m_eventPackage; // Backward compatibility
+                                               : *(unsigned*)&command.m_param.m_registrationInfo.m_eventPackage; // Backward compatibility
 #endif
         subParams.m_restoreTime = command.m_param.m_registrationInfo.m_restoreTime;
         bool ok = sip->Subscribe(subParams, aor);
@@ -1444,15 +1602,17 @@ void OpalManager_C::HandleSetUpCall(const OpalMessage & command, OpalMessageBuff
 
   PString partyA = command.m_param.m_callSetUp.m_partyA;
   if (partyA.IsEmpty()) {
-#if OPAL_PTLIB_AUDIO
 #if OPAL_HAS_PCSS
-    if (pcssEP != NULL)
+    if (m_pcssEP != NULL)
       partyA = "pc:*";
     else
 #endif
-#endif
-    if (localEP != NULL)
+    if (m_localEP != NULL)
       partyA = "local:*";
+#if OPAL_IVR
+    else if (m_ivrEP != NULL)
+      partyA = "ivr:*";
+#endif
     else
       partyA = "pots:*";
   }
@@ -1485,25 +1645,17 @@ void OpalManager_C::HandleAlerting(const OpalMessage & command, OpalMessageBuffe
     return;
   }
 
-  if (
-#if OPAL_PTLIB_AUDIO
 #if OPAL_HAS_PCSS
-      pcssEP == NULL &&
-#endif
-#endif
-      localEP == NULL) {
-    response.SetError("Can only control alerting from PC.");
-    return;
-  }
-
-#if OPAL_PTLIB_AUDIO
-#if OPAL_HAS_PCSS
-  if (pcssEP != NULL && pcssEP->AlertingIncomingCall(command.m_param.m_callToken))
+  if (m_pcssEP != NULL && m_pcssEP->AlertingIncomingCall(command.m_param.m_callToken))
     return;
 #endif
+
+#if OPAL_IVR
+  if (m_ivrEP != NULL && m_ivrEP->AlertingIncomingCall(command.m_param.m_callToken))
+    return;
 #endif
 
-  if (localEP != NULL && localEP->AlertingIncomingCall(command.m_param.m_callToken))
+  if (m_localEP != NULL && m_localEP->AlertingIncomingCall(command.m_param.m_callToken))
     return;
 
   response.SetError("No call found by the token provided.");
@@ -1517,25 +1669,17 @@ void OpalManager_C::HandleAnswerCall(const OpalMessage & command, OpalMessageBuf
     return;
   }
 
-  if (
-#if OPAL_PTLIB_AUDIO
 #if OPAL_HAS_PCSS
-      pcssEP == NULL &&
-#endif
-#endif
-      localEP == NULL) {
-    response.SetError("Can only answer calls to PC.");
-    return;
-  }
-
-#if OPAL_PTLIB_AUDIO
-#if OPAL_HAS_PCSS
-  if (pcssEP != NULL && pcssEP->AcceptIncomingCall(command.m_param.m_callToken))
+  if (m_pcssEP != NULL && m_pcssEP->AcceptIncomingCall(command.m_param.m_callToken))
     return;
 #endif
+
+#if OPAL_IVR
+  if (m_ivrEP != NULL && m_ivrEP->AcceptIncomingCall(command.m_param.m_callToken))
+    return;
 #endif
 
-  if (localEP != NULL && localEP->AcceptIncomingCall(command.m_param.m_callToken))
+  if (m_localEP != NULL && m_localEP->AcceptIncomingCall(command.m_param.m_callToken))
     return;
 
   response.SetError("No call found by the token provided.");
@@ -1660,12 +1804,15 @@ void OpalManager_C::HandleTransferCall(const OpalMessage & command, OpalMessageB
   PSafePtr<OpalConnection> connection = call->GetConnection(0, PSafeReadOnly);
   while (connection->GetLocalPartyURL().NumCompare(search) != EqualTo) {
     if (++connection == NULL) {
-      response.SetError("Call does not have suitable connection to transfer.");
+      response.SetError("Call does not have suitable connection to transfer from " + search);
       return;
     }
   }
 
-  connection->TransferConnection(command.m_param.m_callSetUp.m_partyB);
+  if (connection->GetPhase() < OpalConnection::ConnectedPhase)
+    connection->ForwardCall(command.m_param.m_callSetUp.m_partyB);
+  else
+    call->Transfer(command.m_param.m_callSetUp.m_partyB, connection);
 }
 
 
@@ -1815,6 +1962,36 @@ void OpalManager_C::OnEstablishedCall(OpalCall & call)
 }
 
 
+void OpalManager_C::OnHold(OpalConnection & connection, bool fromRemote, bool onHold)
+{
+  if (fromRemote) {
+    OpalMessageBuffer message(onHold ? OpalIndOnHold : OpalIndOffHold);
+    SET_MESSAGE_STRING(message, m_param.m_callToken, connection.GetCall().GetToken());
+    PostMessage(message);
+  }
+
+  OpalManager::OnHold(connection, fromRemote, onHold);
+}
+
+
+bool OpalManager_C::OnTransferNotify(OpalConnection & connection,
+                                     const PStringToString & info)
+{
+  OpalMessageBuffer message(OpalIndTransferCall);
+
+  SET_MESSAGE_STRING(message, m_param.m_transferStatus.m_callToken, connection.GetCall().GetToken());
+  SET_MESSAGE_STRING(message, m_param.m_transferStatus.m_result, info["result"]);
+
+  PStringStream infoStr;
+  infoStr << info;
+  SET_MESSAGE_STRING(message, m_param.m_transferStatus.m_info, infoStr);
+
+  PostMessage(message);
+
+  return OpalManager::OnTransferNotify(connection, info);
+}
+
+
 void OpalManager_C::OnIndMediaStream(const OpalMediaStream & stream, OpalMediaStates state)
 {
   const OpalConnection & connection = stream.GetConnection();
@@ -2270,6 +2447,7 @@ OpalParamSetUpCall * OpalMessagePtr::GetCallSetUp() const
     case OpalIndProceeding :
     case OpalIndAlerting :
     case OpalIndEstablished :
+    case OpalIndCompletedIVR :
       return &m_message->m_param.m_callSetUp;
 
     default :
@@ -2346,4 +2524,10 @@ OpalParamRecording * OpalMessagePtr::GetRecording() const
 }
 
 
+OpalStatusTransferCall * OpalMessagePtr::GetTransferStatus() const
+{
+  return m_message->m_type == OpalIndTransferCall ? &m_message->m_param.m_transferStatus : NULL;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
