@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23260 $
+ * $Revision: 25780 $
  * $Author: rjongbloed $
- * $Date: 2009-08-24 04:56:39 -0500 (Mon, 24 Aug 2009) $
+ * $Date: 2011-05-16 21:05:05 -0500 (Mon, 16 May 2011) $
  */
 
 #include <ptlib.h>
@@ -75,11 +75,15 @@ PObject::Comparison OpalMediaFormatPair::Compare(const PObject & obj) const
 OpalTranscoder::OpalTranscoder(const OpalMediaFormat & inputMediaFormat,
                                const OpalMediaFormat & outputMediaFormat)
   : OpalMediaFormatPair(inputMediaFormat, outputMediaFormat)
+  , maxOutputSize(32768) // Just something, usually changed by OpalMediaPatch
+  , m_sessionID(0)
+  , outputIsRTP(false)
+  , inputIsRTP(false)
+  , acceptEmptyPayload(false)
+  , acceptOtherPayloads(false)
+  , m_inClockRate(inputMediaFormat.GetClockRate())
+  , m_outClockRate(outputMediaFormat.GetClockRate())
 {
-  maxOutputSize = 32768; // Just something, usually changed by OpalMediaPatch
-  outputIsRTP = inputIsRTP = PFalse;
-  acceptEmptyPayload = false;
-  acceptOtherPayloads = false;
 }
 
 
@@ -87,21 +91,13 @@ bool OpalTranscoder::UpdateMediaFormats(const OpalMediaFormat & input, const Opa
 {
   PWaitAndSignal mutex(updateMutex);
 
-  if (input.IsValid()) {
-    if (inputMediaFormat == input)
-      inputMediaFormat = input;
-    else if (!inputMediaFormat.Merge(input))
-      return false;
-  }
+  bool ok = inputMediaFormat.Update(input);
+  ok = outputMediaFormat.Update(output) && ok; // Avoid McCarthy boolean
 
-  if (output.IsValid()) {
-    if (outputMediaFormat == output)
-      outputMediaFormat = output;
-    else if (!outputMediaFormat.Merge(output))
-      return false;
-  }
+  m_inClockRate = inputMediaFormat.GetClockRate();
+  m_outClockRate = outputMediaFormat.GetClockRate();
 
-  return true;
+  return ok;
 }
 
 
@@ -111,6 +107,15 @@ PBoolean OpalTranscoder::ExecuteCommand(const OpalMediaCommand & /*command*/)
 }
 
 
+void OpalTranscoder::NotifyCommand(const OpalMediaCommand & command) const
+{
+  if (commandNotifier != PNotifier())
+    commandNotifier(const_cast<OpalMediaCommand &>(command), m_sessionID);
+  else
+    PTRACE(4, "Opal\tNo command notifier available for transcoder " << this);
+}
+
+
 void OpalTranscoder::SetInstanceID(const BYTE * /*instance*/, unsigned /*instanceLen*/)
 {
 }
@@ -118,6 +123,7 @@ void OpalTranscoder::SetInstanceID(const BYTE * /*instance*/, unsigned /*instanc
 
 RTP_DataFrame::PayloadTypes OpalTranscoder::GetPayloadType(PBoolean input) const
 {
+  PWaitAndSignal mutex(updateMutex);
   return (input ? inputMediaFormat : outputMediaFormat).GetPayloadType();
 }
 
@@ -129,8 +135,23 @@ void OpalTranscoder::GetStatistics(OpalMediaStatistics & /*statistics*/) const
 #endif
 
 
+void OpalTranscoder::CopyTimestamp(RTP_DataFrame & dst, const RTP_DataFrame & src, bool inToOut) const
+{
+  unsigned timestamp = src.GetTimestamp();
+  if (m_inClockRate != m_outClockRate) {
+    if (inToOut)
+      timestamp = (unsigned)((PUInt64)timestamp*m_outClockRate/m_inClockRate);
+    else
+      timestamp = (unsigned)((PUInt64)timestamp*m_inClockRate/m_outClockRate);
+  }
+  dst.SetTimestamp(timestamp);
+}
+
+
 PBoolean OpalTranscoder::ConvertFrames(const RTP_DataFrame & input, RTP_DataFrameList & output)
 {
+  PWaitAndSignal mutex(updateMutex);
+
   // make sure there is at least one output frame available
   if (output.IsEmpty())
     output.Append(new RTP_DataFrame(0, maxOutputSize));
@@ -140,12 +161,7 @@ PBoolean OpalTranscoder::ConvertFrames(const RTP_DataFrame & input, RTP_DataFram
   }
 
   // set the output timestamp and marker bit
-  unsigned timestamp = input.GetTimestamp();
-  unsigned inClockRate = inputMediaFormat.GetClockRate();
-  unsigned outClockRate = outputMediaFormat.GetClockRate();
-  if (inClockRate != outClockRate)
-    timestamp = (unsigned)((PUInt64)timestamp*outClockRate/inClockRate);
-  output.front().SetTimestamp(timestamp);
+  CopyTimestamp(output.front(), input, true);
   output.front().SetMarker(input.GetMarker());
 
   // set the output payload type directly from the output media format
@@ -171,12 +187,18 @@ OpalTranscoder * OpalTranscoder::Create(const OpalMediaFormat & srcFormat,
                                                        unsigned instanceLen)
 {
   OpalTranscoder * transcoder = OpalTranscoderFactory::CreateInstance(OpalTranscoderKey(srcFormat, destFormat));
-  if (transcoder != NULL) {
-    transcoder->UpdateMediaFormats(srcFormat, destFormat);
-    transcoder->SetInstanceID(instance, instanceLen);
+  if (transcoder == NULL) {
+    PTRACE(2, "Opal\tCould not create transcoder instance from " << srcFormat << " to " << destFormat);
+    return NULL;
   }
 
-  return transcoder;
+  transcoder->SetInstanceID(instance, instanceLen); // Make sure this is done first
+  if (transcoder->UpdateMediaFormats(srcFormat, destFormat))
+    return transcoder;
+
+  delete transcoder;
+  PTRACE(2, "Opal\tError creating transcoder instance from " << srcFormat << " to " << destFormat);
+  return NULL;
 }
 
 
@@ -251,7 +273,8 @@ static bool MergeFormats(const OpalMediaFormatList & masterFormats,
   }
   else {
     srcFormat = *masterFormat;
-    PTRACE(5, "Opal\tInitial source format from master:\n" << setw(-1) << srcFormat);
+    PTRACE(5, "Opal\tInitial source format from master:\n" << setw(-1) << srcFormat
+                        << "Merging with capability\n" << setw(-1) << srcCapability);
     if (!srcFormat.Merge(srcCapability))
       return false;
   }
@@ -263,8 +286,14 @@ static bool MergeFormats(const OpalMediaFormatList & masterFormats,
   }
   else {
     dstFormat = *masterFormat;
-    PTRACE(5, "Opal\tInitial destination format from master:\n" << setw(-1) << dstFormat);
-    if (!dstFormat.Merge(dstCapability))
+    PTRACE(5, "Opal\tInitial destination format from master:\n" << setw(-1) << dstFormat
+                             << "Merging with capability\n" << setw(-1) << dstCapability);
+
+    OpalMediaFormat tmpCapability = dstCapability;
+    if (!tmpCapability.Merge(dstFormat))
+      return false;
+
+    if (!dstFormat.Update(tmpCapability))
       return false;
   }
 
@@ -278,7 +307,8 @@ static bool MergeFormats(const OpalMediaFormatList & masterFormats,
 }
 
 
-bool OpalTranscoder::SelectFormats(const OpalMediaFormatList & srcFormats,
+bool OpalTranscoder::SelectFormats(const OpalMediaType & mediaType,
+                                   const OpalMediaFormatList & srcFormats,
                                    const OpalMediaFormatList & dstFormats,
                                    const OpalMediaFormatList & allFormats,
                                    OpalMediaFormat & srcFormat,
@@ -290,7 +320,7 @@ bool OpalTranscoder::SelectFormats(const OpalMediaFormatList & srcFormats,
   // directly from the given format to a possible one with no transcoders.
   for (d = dstFormats.begin(); d != dstFormats.end(); ++d) {
     for (s = srcFormats.begin(); s != srcFormats.end(); ++s) {
-      if (*s == *d && MergeFormats(allFormats, *s, *d, srcFormat, dstFormat))
+      if (*s == *d && s->GetMediaType() == mediaType && MergeFormats(allFormats, *s, *d, srcFormat, dstFormat))
         return true;
     }
   }
@@ -298,11 +328,13 @@ bool OpalTranscoder::SelectFormats(const OpalMediaFormatList & srcFormats,
   // Search for a single transcoder to get from a to b
   for (d = dstFormats.begin(); d != dstFormats.end(); ++d) {
     for (s = srcFormats.begin(); s != srcFormats.end(); ++s) {
-      OpalTranscoderKey search(*s, *d);
-      OpalTranscoderList availableTranscoders = OpalTranscoderFactory::GetKeyList();
-      for (OpalTranscoderIterator i = availableTranscoders.begin(); i != availableTranscoders.end(); ++i) {
-        if (search == *i && MergeFormats(allFormats, *s, *d, srcFormat, dstFormat))
-          return true;
+      if (s->GetMediaType() == mediaType || d->GetMediaType() == mediaType) {
+        OpalTranscoderKey search(*s, *d);
+        OpalTranscoderList availableTranscoders = OpalTranscoderFactory::GetKeyList();
+        for (OpalTranscoderIterator i = availableTranscoders.begin(); i != availableTranscoders.end(); ++i) {
+          if (search == *i && MergeFormats(allFormats, *s, *d, srcFormat, dstFormat))
+            return true;
+        }
       }
     }
   }
@@ -310,9 +342,12 @@ bool OpalTranscoder::SelectFormats(const OpalMediaFormatList & srcFormats,
   // Last gasp search for a double transcoder to get from a to b
   for (d = dstFormats.begin(); d != dstFormats.end(); ++d) {
     for (s = srcFormats.begin(); s != srcFormats.end(); ++s) {
-      OpalMediaFormat intermediateFormat;
-      if (FindIntermediateFormat(*s, *d, intermediateFormat) && MergeFormats(allFormats, *s, *d, srcFormat, dstFormat))
-        return true;
+      if (s->GetMediaType() == mediaType || d->GetMediaType() == mediaType) {
+        OpalMediaFormat intermediateFormat;
+        if (FindIntermediateFormat(*s, *d, intermediateFormat) &&
+            MergeFormats(allFormats, *s, *d, srcFormat, dstFormat))
+          return true;
+      }
     }
   }
 
@@ -405,22 +440,10 @@ OpalMediaFormatList OpalTranscoder::GetPossibleFormats(const OpalMediaFormatList
 /////////////////////////////////////////////////////////////////////////////
 
 OpalFramedTranscoder::OpalFramedTranscoder(const OpalMediaFormat & inputMediaFormat,
-                                           const OpalMediaFormat & outputMediaFormat,
-                                           PINDEX inputBytes, PINDEX outputBytes)
+                                           const OpalMediaFormat & outputMediaFormat)
   : OpalTranscoder(inputMediaFormat, outputMediaFormat)
 {
-  PINDEX framesPerPacket = outputMediaFormat.GetOptionInteger(OpalAudioFormat::TxFramesPerPacketOption(), 1);
-  inputBytesPerFrame = inputBytes*framesPerPacket;
-  outputBytesPerFrame = outputBytes*framesPerPacket;
-
-  PINDEX inMaxTimePerFrame  = inputMediaFormat.GetOptionInteger(OpalAudioFormat::MaxFramesPerPacketOption()) * 
-                              inputMediaFormat.GetOptionInteger(OpalAudioFormat::FrameTimeOption());
-  PINDEX outMaxTimePerFrame = outputMediaFormat.GetOptionInteger(OpalAudioFormat::MaxFramesPerPacketOption()) * 
-                              outputMediaFormat.GetOptionInteger(OpalAudioFormat::FrameTimeOption());
-
-  PINDEX maxPacketTime = PMAX(inMaxTimePerFrame, outMaxTimePerFrame);
-
-  maxOutputDataSize = outputBytesPerFrame * (maxPacketTime / outputMediaFormat.GetOptionInteger(OpalAudioFormat::FrameTimeOption()));
+  CalculateSizes();
 }
 
 
@@ -435,7 +458,15 @@ bool OpalFramedTranscoder::UpdateMediaFormats(const OpalMediaFormat & input, con
   if (!OpalTranscoder::UpdateMediaFormats(input, output))
     return false;
 
-  unsigned framesPerPacket = outputMediaFormat.GetOptionInteger(OpalAudioFormat::TxFramesPerPacketOption(), 1);
+  CalculateSizes();
+  return true;
+}
+
+
+void OpalFramedTranscoder::CalculateSizes()
+{
+  unsigned framesPerPacket = outputMediaFormat.GetOptionInteger(OpalAudioFormat::TxFramesPerPacketOption(),
+                              inputMediaFormat.GetOptionInteger(OpalAudioFormat::TxFramesPerPacketOption(), 1));
   unsigned inFrameSize = inputMediaFormat.GetFrameSize();
   unsigned outFrameSize = outputMediaFormat.GetFrameSize();
   unsigned inFrameTime = inputMediaFormat.GetFrameTime();
@@ -452,8 +483,6 @@ bool OpalFramedTranscoder::UpdateMediaFormats(const OpalMediaFormat & input, con
   PINDEX maxPacketTime = PMAX(inMaxTimePerFrame, outMaxTimePerFrame);
 
   maxOutputDataSize = outputBytesPerFrame * (maxPacketTime / outputMediaFormat.GetOptionInteger(OpalAudioFormat::FrameTimeOption()));
-
-  return true;
 }
 
 
@@ -465,6 +494,8 @@ PINDEX OpalFramedTranscoder::GetOptimalDataFrameSize(PBoolean input) const
 
 PBoolean OpalFramedTranscoder::Convert(const RTP_DataFrame & input, RTP_DataFrame & output)
 {
+  // Note updateMutex should already be locked at this point.
+
   if (inputIsRTP || outputIsRTP) {
 
     const BYTE * inputPtr;
@@ -493,7 +524,7 @@ PBoolean OpalFramedTranscoder::Convert(const RTP_DataFrame & input, RTP_DataFram
     }
 
     if (!ConvertFrame(inputPtr, inLen, outputPtr, outLen))
-      return PFalse;
+      return false;
 
     if (!outputIsRTP)
       output.SetPayloadSize(outLen);
@@ -504,7 +535,7 @@ PBoolean OpalFramedTranscoder::Convert(const RTP_DataFrame & input, RTP_DataFram
     else 
       output.SetPayloadSize(outLen - output.GetHeaderSize());
 
-    return PTrue;
+    return true;
   }
 
   const BYTE * inputPtr = input.GetPayloadPtr();
@@ -517,20 +548,20 @@ PBoolean OpalFramedTranscoder::Convert(const RTP_DataFrame & input, RTP_DataFram
 
   // set maximum output payload size
   if (!output.SetPayloadSize(maxOutputDataSize))
-    return PFalse;
+    return false;
 
   BYTE * outputPtr = output.GetPayloadPtr();
-
   PINDEX outLen = 0;
 
-  while (inputLength > 0) {
+  while (inputLength > 0 && outLen < maxOutputDataSize) {
 
-    PINDEX consumed = inputLength; // PMIN(inputBytesPerFrame, inputLength);
-    PINDEX created  = output.GetPayloadSize() - outLen;
+    PINDEX consumed = inputLength;
+    PINDEX created  = maxOutputDataSize - outLen;
 
     if (!ConvertFrame(inputPtr, consumed, outputPtr, created))
-      return PFalse;
+      return false;
 
+    // If did not consume or produce any data, codec has gone wrong, abort!
     if (consumed == 0 && created == 0)
       break;
 
@@ -543,7 +574,7 @@ PBoolean OpalFramedTranscoder::Convert(const RTP_DataFrame & input, RTP_DataFram
   // set actual output payload size
   output.SetPayloadSize(outLen);
 
-  return PTrue;
+  return true;
 }
 
 PBoolean OpalFramedTranscoder::ConvertFrame(const BYTE * inputPtr, PINDEX & /*consumed*/, BYTE * outputPtr, PINDEX & /*created*/)
@@ -578,8 +609,10 @@ OpalStreamedTranscoder::OpalStreamedTranscoder(const OpalMediaFormat & inputMedi
 PINDEX OpalStreamedTranscoder::GetOptimalDataFrameSize(PBoolean input) const
 {
   // For streamed codecs a "frame" is one milliseconds worth of data
-  PINDEX size = outputMediaFormat.GetOptionInteger(input ? OpalAudioFormat::TxFramesPerPacketOption()
-                                                         : OpalAudioFormat::RxFramesPerPacketOption(), 1);
+  PString framesPerPacketOption = input ? OpalAudioFormat::TxFramesPerPacketOption()
+                                        : OpalAudioFormat::RxFramesPerPacketOption();
+  PINDEX size = outputMediaFormat.GetOptionInteger(framesPerPacketOption,
+                 inputMediaFormat.GetOptionInteger(framesPerPacketOption, 1));
 
   size *= outputMediaFormat.GetClockRate()/1000;            // Convert to milliseconds
   size *= input ? inputBitsPerSample : outputBitsPerSample; // Total bits
@@ -595,7 +628,7 @@ PBoolean OpalStreamedTranscoder::Convert(const RTP_DataFrame & input,
   PINDEX i, bit, mask;
 
   PINDEX samples = input.GetPayloadSize()*8/inputBitsPerSample;
-  PINDEX outputSize = samples*outputBitsPerSample/8;
+  PINDEX outputSize = (samples * outputBitsPerSample + 7) / 8;
   output.SetPayloadSize(outputSize);
 
   // The conversion algorithm for 5,3 & 2 bits per sample needs an extra
