@@ -28,9 +28,9 @@
  *     US Joint Forces Command Joint Concept Development & Experimentation (J9)
  *     http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision: 23294 $
+ * $Revision: 25956 $
  * $Author: rjongbloed $
- * $Date: 2009-08-28 01:26:26 -0500 (Fri, 28 Aug 2009) $
+ * $Date: 2011-06-01 21:39:26 -0500 (Wed, 01 Jun 2011) $
  */
 
 #include <ptlib.h>
@@ -132,8 +132,8 @@ PBoolean OpalTransportAddress::IsCompatible(const OpalTransportAddress & address
   if (IsEmpty() || address.IsEmpty())
     return PTrue;
 
-  PCaselessString myPrefix = GetProto();
-  PCaselessString theirPrefix = address.GetProto();
+  PCaselessString myPrefix = GetProto(true);
+  PCaselessString theirPrefix = address.GetProto(true);
   return myPrefix == theirPrefix ||
         (myPrefix    == IpPrefix && (theirPrefix == TcpPrefix || theirPrefix == UdpPrefix 
 #if OPAL_PTLIB_SSL
@@ -295,29 +295,72 @@ PBoolean OpalInternalTransport::GetIpAndPort(const OpalTransportAddress &,
 static PBoolean SplitAddress(const PString & addr, PString & host, PString & device, PString & service)
 {
   // skip transport identifier
-  PINDEX dollar = addr.Find('$');
-  if (dollar == P_MAX_INDEX)
+  PINDEX i = addr.Find('$');
+  if (i == P_MAX_INDEX) {
+    //PTRACE(1, "Address " << addr << " has no dollar");
     return false;
+  }
+
+  host.MakeEmpty();
+  device.MakeEmpty();
+  service.MakeEmpty();
+
+  PINDEX j = ++i;
+
+  // parse interface/host
+  bool isInterface = (addr[j] == '%') || (addr[j] == '[' && addr[j+1] == '%');
+
+  if (j == '\0') {
+    //PTRACE(1, "Address " << addr << " has empty host");
+    return false;
+  }
+  i = j;
+  bool bracketed = addr[j] == '[';
+  for (;;) {
+    if (addr[j] == '\0')
+      break;
+    if (!bracketed) {
+      if (addr[j] == ':')
+        break;
+    } else if (addr[j] == ']') {
+      ++j;
+      break;
+    }
+    j++;
+  }
+  if (i == j) {
+    //PTRACE(1, "Address " << addr << " has invalid host " << i);
+    return false;
+  }
+
+  if (!isInterface)
+    host = addr(i, j-1);
+  else {
+    if (addr[i] == '[' && addr[i+1] == '%') {
+      device = '%';
+      device += addr(i+2, j-2);
+    }
+    else
+      device = addr(i, j-1);
+  }
   
-  PINDEX length = addr.GetLength();
-  if (addr[length-1] == '+')
-    length--;
+  // parse optional service
+  if (addr[j] == ':') {
+    i = ++j;
+    for (;;) {
+      if (addr[j] == '\0')
+        break;
+      j++;
+    }
+    // cannot have zero length service
+    if (i == j) {
+      //PTRACE(1, "Address " << addr << " has invalid service " << i << " " << j);
+      return false;
+    }
+    service = addr(i, j-1);
+  }
 
-  PINDEX bracket = addr.FindLast(']');
-  if (bracket == P_MAX_INDEX)
-    bracket = 0;
-
-  PINDEX colon = addr.FindLast(':');
-  if (colon == P_MAX_INDEX || colon < bracket)
-    colon = length;
-
-  PINDEX percent = addr.Find('%', dollar);
-  if (percent == P_MAX_INDEX)
-    percent = colon;
-
-  host = addr(dollar+1, percent-1);
-  device = addr(percent, colon-1);
-  service = addr(colon+1, length-1);
+  //PTRACE(1, "Split " << addr << " into host='" << host << "',dev='" << device << "',service='" << service << "'");
 
   return true;
 }
@@ -368,15 +411,21 @@ PBoolean OpalInternalIPTransport::GetIpAndPort(const OpalTransportAddress & addr
     }
   }
 
-  if (host[0] == '*' || host == "0.0.0.0"
-#ifdef P_HAS_IPV6
-	  || host == ":" || host == "::"  || host == "[::]"
-#endif
-	  ) {
+  if (host[0] == '*') {
     ip = PIPSocket::GetDefaultIpAny();
     return PTrue;
   }
 
+  if (host == "0.0.0.0") {
+    ip = PIPSocket::Address::GetAny(4);
+    return true;
+  }
+
+  if (host == "::" || host == "[::]") {
+    ip = PIPSocket::Address::GetAny(6);
+    return true;
+  }
+
   if (device.IsEmpty()) {
     if (PIPSocket::GetHostAddress(host, ip))
       return true;
@@ -679,8 +728,9 @@ OpalListenerUDP::OpalListenerUDP(OpalEndPoint & endpoint,
 OpalListenerUDP::OpalListenerUDP(OpalEndPoint & endpoint,
                                  const OpalTransportAddress & binding,
                                  OpalTransportAddress::BindOptions option)
-  : OpalListenerIP(endpoint, binding, option),
-    listenerBundle(PMonitoredSockets::Create(binding.GetHostName(), !exclusiveListener, endpoint.GetManager().GetNatMethod()))
+  : OpalListenerIP(endpoint, binding, option)
+  , listenerBundle(PMonitoredSockets::Create(binding.GetHostName(), !exclusiveListener, endpoint.GetManager().GetNatMethod()))
+  , m_bufferSize(32768)
 {
 }
 
@@ -730,21 +780,31 @@ OpalTransport * OpalListenerUDP::Accept(const PTimeInterval & timeout)
   WORD remotePort;
   PString iface;
   PINDEX readCount;
-  static const PINDEX SixtyFourK = 0x10000;
-  switch (listenerBundle->ReadFromBundle(pdu.GetPointer(SixtyFourK), SixtyFourK, remoteAddr, remotePort, iface, readCount, timeout)) {
+  bool preReadOK;
+  switch (listenerBundle->ReadFromBundle(pdu.GetPointer(m_bufferSize), m_bufferSize, remoteAddr, remotePort, iface, readCount, timeout)) {
     case PChannel::NoError :
       pdu.SetSize(readCount);
-      return new OpalTransportUDP(endpoint, pdu, listenerBundle, iface, remoteAddr, remotePort);
+      preReadOK = true;
+      break;
+
+    case PChannel::BufferTooSmall :
+      preReadOK = false;
+      break;
 
     case PChannel::Interrupted :
       PTRACE(4, "Listen\tInterfaces changed");
-      break;
+      return NULL;
 
     default :
       PTRACE(1, "Listen\tUDP read error.");
+      return NULL;
   }
 
-  return NULL;
+  OpalTransportUDP * transport = new OpalTransportUDP(endpoint, listenerBundle, iface);
+  transport->m_preReadPacket = pdu;
+  transport->m_preReadOK = preReadOK;
+  transport->SetRemoteAddress(OpalTransportAddress(remoteAddr, remotePort, "udp"));
+  return transport;
 }
 
 
@@ -755,14 +815,11 @@ OpalTransport * OpalListenerUDP::CreateTransport(const OpalTransportAddress & lo
     return NULL;
 
   PIPSocket::Address addr;
-  if (remoteAddress.GetIpAddress(addr) && addr.IsLoopback())
-    return new OpalTransportUDP(endpoint, addr);
-
   PString iface;
   if (localAddress.GetIpAddress(addr))
     iface = addr.AsString(true);
 
-  return new OpalTransportUDP(endpoint, PBYTEArray(), listenerBundle, iface, PIPSocket::GetDefaultIpAny(), 0);
+  return new OpalTransportUDP(endpoint, listenerBundle, iface);
 }
 
 
@@ -1164,6 +1221,8 @@ OpalTransportUDP::OpalTransportUDP(OpalEndPoint & ep,
                                    bool preOpen)
   : OpalTransportIP(ep, binding, localPort)
   , manager(ep.GetManager())
+  , m_bufferSize(8192)
+  , m_preReadOK(false)
 {
   PMonitoredSockets * sockets = PMonitoredSockets::Create(binding.AsString(), reuseAddr, manager.GetNatMethod());
   if (preOpen)
@@ -1173,20 +1232,14 @@ OpalTransportUDP::OpalTransportUDP(OpalEndPoint & ep,
 
 
 OpalTransportUDP::OpalTransportUDP(OpalEndPoint & ep,
-                                   const PBYTEArray & packet,
                                    const PMonitoredSocketsPtr & listener,
-                                   const PString & iface,
-                                   PIPSocket::Address remAddr,
-                                   WORD remPort)
+                                   const PString & iface)
   : OpalTransportIP(ep, PIPSocket::GetDefaultIpAny(), 0)
   , manager(ep.GetManager())
-  , preReadPacket(packet)
+  , m_bufferSize(8192)
+  , m_preReadOK(true)
 {
-  remoteAddress = remAddr;
-  remotePort = remPort;
-
   PMonitoredSocketChannel * socket = new PMonitoredSocketChannel(listener, PTrue);
-  socket->SetRemote(remAddr, remPort);
   socket->SetInterface(iface);
   socket->GetLocal(localAddress, localPort, !manager.IsLocalAddress(remoteAddress));
   Open(socket);
@@ -1362,28 +1415,28 @@ PString OpalTransportUDP::GetLastReceivedInterface() const
 
 PBoolean OpalTransportUDP::Read(void * buffer, PINDEX length)
 {
-  if (preReadPacket.IsEmpty())
+  if (m_preReadPacket.IsEmpty())
     return OpalTransportIP::Read(buffer, length);
 
-  lastReadCount = PMIN(length, preReadPacket.GetSize());
-  memcpy(buffer, preReadPacket, lastReadCount);
-  preReadPacket.SetSize(0);
+  lastReadCount = PMIN(length, m_preReadPacket.GetSize());
+  memcpy(buffer, m_preReadPacket, lastReadCount);
+  m_preReadPacket.SetSize(0);
 
-  return PTrue;
+  return m_preReadOK;
 }
 
 
 PBoolean OpalTransportUDP::ReadPDU(PBYTEArray & packet)
 {
-  if (preReadPacket.GetSize() > 0) {
-    packet = preReadPacket;
-    preReadPacket.SetSize(0);
-    return PTrue;
+  if (m_preReadPacket.GetSize() > 0) {
+    packet = m_preReadPacket;
+    m_preReadPacket.SetSize(0);
+    return m_preReadOK;
   }
 
-  if (!Read(packet.GetPointer(10000), 10000)) {
+  if (!Read(packet.GetPointer(m_bufferSize), m_bufferSize)) {
     packet.SetSize(0);
-    return PFalse;
+    return false;
   }
 
   packet.SetSize(GetLastReadCount());
@@ -1410,10 +1463,16 @@ PBoolean OpalTransportUDP::WriteConnect(WriteConnectCallback function, void * us
 
   PBoolean ok = PFalse;
   for (PINDEX i = 0; i < interfaces.GetSize(); i++) {
-    PTRACE(4, "OpalUDP\tWriting to interface " << i << " - \"" << interfaces[i] << '"');
-    socket->SetInterface(interfaces[i]);
-    if (function(*this, userData))
-      ok = PTrue;
+    PIPSocket::Address ifip(interfaces[i]);
+    if (ifip.GetVersion() != remoteAddress.GetVersion())
+      PTRACE(4, "OpalUDP\tSkipping incompatible interface " << i << " - \"" << interfaces[i] << '"');
+    else {
+      PTRACE(4, "OpalUDP\tWriting to interface " << i << " - \"" << interfaces[i] << '"');
+      socket->SetInterface(interfaces[i]);
+      // Make sure is compatible address
+      if (function(*this, userData))
+        ok = PTrue;
+    }
   }
 
   return ok;
